@@ -46,6 +46,7 @@ class TextractBlockTypes(Enum):
     QUERY = auto()
     QUERY_RESULT = auto()
     MERGED_CELL = auto()
+    SIGNATURE = auto()
 
 
 @dataclass
@@ -456,6 +457,7 @@ class TResponseMetadata():
 @dataclass(eq=True, init=True, repr=True)
 class TDocument():
     document_metadata: TDocumentMetadata = field(default=None)  # type: ignore
+    # if blocks are changed, call __post_init__() to update the index
     blocks: List[TBlock] = field(default=None)  # type: ignore
     analyze_document_model_version: str = field(default=None)  # type: ignore
     detect_document_text_model_version: str = field(
@@ -540,6 +542,7 @@ class TDocument():
         if not page:
             page = self.pages[0]
         page.add_ids_to_relationships(ids=[block.id])
+        self.__post_init__()
         self.relationships_recursive.cache_clear()
 
     @staticmethod
@@ -644,10 +647,11 @@ class TDocument():
         return None
 
     def get_block_by_id(self, id: str) -> TBlock:
-        for b in self.blocks:
-            if b.id == id:
-                return b
-        raise ValueError(f"no block for id: {id}")
+        block = self.find_block_by_id(id=id)
+        if block:
+            return block
+        else:
+            raise ValueError(f"no block for id: {id}")
 
     # Sometimes Textract returns a block with an id in "Relationships" that is not present in the flattened list of blocks.
     # This is a Textract bug upstream of TRP workaround for that.
@@ -725,9 +729,9 @@ class TDocument():
 
     def keys(self, page: TBlock = None) -> List[TBlock]:  # type: ignore
         return [x for x in self.forms(page=page) if TextractEntityTypes.KEY.name in x.entity_types]
-        # for key_entities in self.forms(page=page):
-        #     if TextractEntityTypes.KEY.name in key_entities.entity_types:
-        #         yield key_entities
+
+    def signatures(self, page: TBlock) -> List[TBlock]:
+        return self.get_blocks_by_type(page=page, block_type_enum=TextractBlockTypes.SIGNATURE)
 
     def queries(self, page: TBlock) -> List[TBlock]:
         return self.get_blocks_by_type(page=page, block_type_enum=TextractBlockTypes.QUERY)
@@ -794,12 +798,11 @@ class TDocument():
         return self.get_blocks_by_type(page=page, block_type_enum=TextractBlockTypes.LINE)
 
     def delete_blocks(self, block_id: List[str]):
-        for b in block_id:
-            block = self.get_block_by_id(b)
-            if block and self.blocks:
-                self.blocks.remove(block)
-            else:
-                logger.warning(f"delete_blocks: did not get block for id: {b}")
+        # delete from high index number to low index number to avoid deleting the wrong index after removing a lower valued one
+        indexes = [self.block_id_map()[id] for id in block_id]
+        indexes.sort(reverse=True)
+        for index in indexes:
+            del self.blocks[index]
         self.__post_init__()
         self.relationships_recursive.cache_clear()
 
